/*
 * This class implements a RMI bandwidth gauge.
 *
 */

package edu.cmu.cs.able.gauges.RMI;
import java.rmi.*;
import java.util.*;

import java.net.*;
import java.io.*;

import edu.cmu.cs.able.gaugeInfrastructure.*;
import edu.cmu.cs.able.gaugeInfrastructure.RMI.*;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.probes.RemosProbe.*;

public class RMIBandwidthGauge extends RMIGauge implements edu.cmu.cs.able.probes.RemosProbe.BandwidthProbeConsumer {

    private BandwidthProbeConsumer remosProbeConsumer;
    private ProbeManagerHandle pmh;
    private ProbeHandle prbHndl;
    
    // Setup parameters
    /** The IP address of one endpoint to determine the bandwidth between.
     */    
    protected InetAddress ip1;
    
    /** The IP address of one endpoint to determine the bandwidth between.
     */    
    protected InetAddress ip2;
    
    /** How often this gauge should report the bandwidth.
     */    
    protected static int frequency;
    
    /** The bandwidth that was currently measured from the probe.
     */    
    protected Double currentBW = null;
    
    /** The location of the siena bus server.
     */    
    public static String sienaHost = null;

    /** Creates new RMIBandwidthGauge
     * @param gaugeID The ID of the gauge to create.
     * @param creatorID The ID of the gauge manager that is creating the gauge.
     * @param setupParams The value of the setup params.
     */
	public RMIBandwidthGauge (GaugeID gaugeID, String creatorID, StringPairVector setupParams)
	{
		super(gaugeID, creatorID, setupParams);
       // setup default values
        try {
            ip1 = InetAddress.getByName ("localhost");
            ip2 = InetAddress.getByName ("localhost");
            frequency = 500;
        }
        catch (Exception e) {
            ip1 = null;
            ip2 = null;
        }
		
		System.out.println (gaugeID.toString () + " created");

		boolean freq_defined = false;
		boolean ip1_defined = false;
		boolean ip2_defined = false;
		
		try {
        	if (setupParams != null) {
        		for (int i = 0; i < setupParams.size (); i++) {
            		if (setupParams.nameAt (i).equals ("frequency")) {
	                	frequency = Integer.parseInt (setupParams.valueAt (i));
	                	freq_defined = true;
    	        	}
                	else if (setupParams.nameAt(i).equals("IP_addr1")) {
                    	ip1 = InetAddress.getByName(setupParams.valueAt(i));
                    	ip1_defined = true;
                	}
                	else if (setupParams.nameAt(i).equals("IP_addr2")) {
                    	ip2 = InetAddress.getByName(setupParams.valueAt(i));
                    	ip2_defined = true;
                	}
        		}
		        System.out.println ("with");
    		    for (int i = 0; i < setupParams.size (); i ++) {
        		    System.out.println ("  setup " + setupParams.nameAt (i) + " = " + setupParams.valueAt (i));
	        	}
	        }
	    }
	    catch (Exception e) {}
	    
        if (!freq_defined) {
        	configParams.addElement ("frequency", new Integer (frequency).toString ());
        }
        if (!ip1_defined) {
        	configParams.addElement ("IP_addr1", ip1.toString ());
        }
		if (!ip2_defined) {
        	configParams.addElement ("IP_addr2", ip2.toString ());
        }
        
		if (!deployProbes()) {
			System.err.println("Failed to deploy probes");
		}
	}

    private boolean deployProbes () {
        boolean probesDeployed = true;
        pmh = new ProbeManagerHandle (this, "myHandle", sienaHost);
        prbHndl = pmh.deploy ();
        if (prbHndl == null || !prbHndl.activate (ip1, ip2, 1000/frequency)) {
            probesDeployed = false;
        }
        
        return probesDeployed;
    }

    // Probe consumer interface
    static final long EightMega = 1024*1024/8;
    
    /** The callback that implements BandwidthProbeConsumer. This is called when
     * the RemosProbe reports a value.
     * @param bw The bandwidth reported by the RemosProbe.
     */    
    public void sensed (double bw) {
        currentBW = new Double (bw/EightMega);
        gaugeBus.reportValue (getGaugeID (), "bandwidth", "", currentBW.toString ());
    }


    public void reportNewValue (){
    }

    /** Returns the state of the gauge.
     * @param setupParams The vector into which the setup parameters are copied. Will be
     * empty for this gauge.
     * @param configParams The configuration parameters for this gauge. Will have
     * an element for <pre>frequency</pre> and <pre>report</pre>.
     * @param mappings The property that each gauge value is associated with.
     * Will have an element for <pre>date</pre>.
     * @return Whether the state of the gauge could be queried.
     */    
    public boolean queryState(StringPairVector setupParams, StringPairVector configParams, StringPairVector mappings) {
        this.setupParams.copyInto (setupParams);
        configParams.addElement ("frequency", new Integer (frequency).toString ());
		configParams.addElement ("IP_addr1", ip1.toString ());
		configParams.addElement ("IP_addr2", ip2.toString ());
        
        return true;
    }

    /** This method is used to configure the gauge.
     * @param configParams The parameters used to configure the gauge. For the
     * SienaTimeGauge, the valid configuration parameters are:<br>
     * &nbsp;&nbsp;frequency : int (millisecond period of reporting)<br>
     * &nbsp;&nbsp;report : true (whether the gauge should report a value)
     * @return Whether the gauge was successfully configured.
      */    
    public boolean configure(StringPairVector configParams) {
        // The only valid configuration parameter is "Frequency"
        // This value will focus the remos probe to report with this frequency -
        // the gauge will report whenever there is a value sensed
        
        boolean validConfiguration = true;
        Integer frequency = new Integer (configParams.getValue ("frequency"));
        if (frequency.intValue () <= 0) {
            validConfiguration = false;
        }
        else {
            this.frequency = frequency.intValue ();
            validConfiguration = prbHndl.focus (ip1, ip2, 1000/frequency.intValue ());
        }
        
        return validConfiguration;
    }

    /** A command-line interface to start the gauge.
     * @param args -gt <i>gauge type</i><br>
     * -gn <i>gauge name</i><br>
     * -mt <i>model type</i><br>
     * -mn <i>model name</i><br>
     * [-setup <i>name=value</i>]<br>
     * [-mappings <i>name=property</i>]<br>
     * -creator <i>creator ID</i><br>
     */    
	public static void main(String args[])
	{

		String gaugeName = "RMIBandwidthGaugeName";   
		String gaugeType = "edu.cmu.cs.able.gauges.RMI.RMIBandwidthGauge";   
		String modelName = "RMIBandwidthGaugeModelName";   
		String modelType = "RMIBandwidthGaugeModelType";   				    
        String creatorID = "";

        StringPairVector mappings = new StringPairVector ();
        StringPairVector setupParams = new StringPairVector ();
        boolean gotMappings = false;
        boolean gotSetup = false;
        // Process args
        boolean ok = true;
        for (int i = 0; i < args.length && ok; i++) {
            String tag = args [i];
            if (args [i].equals ("-creator")) {
                creatorID = args [++i];
            }
            else if (args [i].equals ("-gn")) {
                gaugeName = args [++i];
            }
            else if (args [i].equals ("-gt")) {
                gaugeType = args [++i];
            }
            else if (args [i].equals ("-mn")) {
                modelName = args [++i];
            }
            else if (args [i].equals ("-mt")) {
                modelType = args [++i];
            }
            else if (args [i].equals ("-setup")) {
                gotSetup = true;
                i++;
                while (i < args.length && args [i].charAt (0) != '-') {
                    String setup = args [i];
                    String name = setup.substring (0, setup.indexOf ('='));
                    String value = setup.substring (setup.indexOf ('=') + 1);
                    setupParams.addElement (name, value);
                    i++;
                } 
                i--;
            }
            else if (args [i].equals ("-mappings")) {
                gotMappings = true;
                i++;
                while (i < args.length && args [i].charAt (0) != '-') {
                    String mapping = args [i];
                    String value = mapping.substring (0, mapping.indexOf ('='));
                    String property = mapping.substring (mapping.indexOf ('=') + 1);
                    mappings.addElement (value, property);
                    i++;
                }
                i--;
            }
            else if (args [i].equals ("-rmib")) {
            	++i;
            }
			else if (args [i].equals ("-senp")) {
                sienaHost = args [++i];
            }

            else {
                ok = false;
            }            
        }
        
        if (!ok) {
            printUsageMessage ();
            System.exit (0);
        }
        
        GaugeID gid = new GaugeID ();
        gid.modelName = modelName;
        gid.modelType = modelType;
        gid.gaugeName = gaugeName;
        gid.gaugeType = gaugeType;

   		System.out.println("RMIBandwidthGauge started");
   		System.out.flush();

   		RMIBandwidthGauge theTimeGauge = new RMIBandwidthGauge(gid, creatorID, setupParams);
   		
   		while (true) {
   			//System.err.println("reported");
   			try {
   				Thread.sleep(frequency);
   			}
   			catch (Exception e) 
   			{
   				e.printStackTrace();
   			}
   		}
	}
    
    private static void printUsageMessage () {
        System.out.println ("Usage: RMIBandwidthGauge -creator <creatorID> -mt <modelType> -mn <modelName> -gt <gaugeType> -gn <gaugeName> -setup <name=value> -mappings <value=property> -rmib <rmibus> -senp <siena port>");
    }
    
}
