/*
 * SienaLatencyGauge.java
 *
 * Created on November 19, 2001, 4:43 PM
 */

package edu.cmu.cs.able.gauges.Siena.Demo;

import java.util.Enumeration;
import edu.cmu.cs.able.gaugeInfrastructure.Siena.*;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.gaugeInfrastructure.Events.*;
import edu.cmu.cs.able.probebus.*;
import edu.cmu.cs.able.probebus.siena.*;
import edu.cmu.cs.able.probebus.events.*;
import edu.cmu.cs.able.probebus.siena.events.*;

/** Implementation of the LatencyGauge, conforming to gauge type:<br>
 * LatencyGauge = [<br>
 * &nbsp;&nbsp;type = edu.cmu.cs.able.gauges.Siena.Demo.LatencyGauge<br>
 * &nbsp;&nbsp;setup = [where : String]<br>
 * &nbsp;&nbsp;config = [reset : boolean]<br>
 * &nbsp;&nbsp;values = [latency : float]
 * @author Bradley Schmerl
 * @version 1.0
 */
public class LatencyGauge extends edu.cmu.cs.able.gaugeInfrastructure.Siena.SienaGauge 
        implements edu.cmu.cs.able.probebus.IProbeConsumer {

    protected String where = null;
    public static final Double ZERO = new Double (0.0);
    Double latency = ZERO;
 
    final double ALPHA = 0.09;
    
    public String gaugeType = "edu.cmu.cs.able.gauges.Siena.Demo.LatencyGauge";
    
    private static IProbeBus probeBus = null;
    private final String CONFIG_NAME = "LatencyConfig";
    private final String TARGET_CLASSES = "runtime.client.Client,runtime.client.ClientThread";
    private KXProbeHelper latencyProbe;
    private IProbeEventFactory probeEventFactory = null;
    /** Creates new SienaLatencyGauge     
     * @param gaugeID The ID of the new gauge.
     * @param creatorID The ID of the gauge manager that created the gauge.
     * @param setupParams The parameters that the gauge uses to set itself up. The
     * valid setup parameters for this gauge are:<br>
     * &nbsp;&nbsp;where : int
     * @param mappings The property that the <pre>latency</pre> value is associated with.
     */
    public LatencyGauge (GaugeID gaugeID, String creatorID, StringPairVector setupParams, StringPairVector mappings) {
        super (gaugeID, creatorID, setupParams, mappings);
        
        for (int i = 0; i < setupParams.size (); i++) {
            if (setupParams.nameAt (i).equals ("where")) {
                where = setupParams.valueAt (i);
            }
        }
        
        probeEventFactory = new SienaProbeEventFactory( (SienaProbeBus )probeBus );
        
        sleepTime = 3000;
        doReporting = false; // Gauge will report when it receives an event from a probe, so
        // this is false - should be renamed to doPeriodicReporting
        latencyProbe = new KXProbeHelper (CONFIG_NAME, where, TARGET_CLASSES, this, probeBus);
        
/*        finalizeCreation (creatorID, deployed);
        
        if (deployed) {
            IProbeEvent e = eventFactory.createLatencyEvent( latency, CONFIG_NAME, where ); 
            probeBus.registerForEvent (e, this);
        }
        */
    }
    
    private void probesDeployed () {
        finalizeCreation (myManagerID, true);
        IProbeEvent e = probeEventFactory.createLatencyEvent( "", CONFIG_NAME, where ); 
        probeBus.registerForEvent (e, this);
    }

    
    /** This method is used to configure the gauge.
     * @param configParams The parameters used to configure the gauge. For the
     * LatencyGauge, it will be [reset : boolean].
     * @return Whether the gauge was successfully configured. Will always return false.
      */    
    public boolean configure(StringPairVector configParams) {
        for (int i = 0; i < configParams.size (); i++) {
            if (configParams.nameAt (i).equals ("reset") &&
                configParams.valueAt (i).equals ("true")) {
                latency = ZERO;
            }
        }
        return true;
    }
    
    /** Returns the current date.
     * @param values The vector into which to place the date.
     * @return Whether the gauge could report it's current value.
     */    
    public boolean queryAllValues (GaugeValueVector values) {
        synchronized (latency) {
            
            values.addElement ("latency", (String )mappings.get ("latency"), latency.toString ());
        }
        return true;
    }
    
    /** Returns the state of the gauge.
     * @param setupParams The vector into which the setup parameters are copied. Will be
     * empty for this gauge.
     * @param configParams The configuration parameters for this gauge. Will have
     * an element for <pre>frequency</pre> and <pre>report</pre>.
     * @param mappings The property that each gauge value is associated with.
     * Will have an element for <pre>date</pre>.
     * @return Whether the state of the gauge could be queried.
     */    
    public boolean queryState(StringPairVector setupParams, StringPairVector configParams, StringPairVector mappings) {
        this.setupParams.copyInto (setupParams);
        configParams.removeAllElements ();
        
        java.util.Enumeration enum = this.mappings.keys ();
        while (enum.hasMoreElements ()) {
            String key = (String )enum.nextElement ();
            mappings.addElement (key, (String )this.mappings.get (key));
        }
        
        return true;
    }
    
    /** The method called to query the value of the gauge. This
     * provides a direct query for the gauge, as opposed to the
     * gauge reporting to value on the bus.
     * @param valueName The name of the value to query.
     * @return The corresponding value of the gauge. (Note: <I>it is assumed
     * that the querier know the property associated with this
     * value</I>).
     */    
    public String queryValue(String valueName) {
        if (valueName.equals ("latency")) {
            String l = "";
            synchronized (latency) {
                l = latency.toString ();
            }
            return l;
        }
        else {
            return "";
        }
    }
    

    /** This method is called when a probe senses a value. It will generate a report when this is called.
     * @param latency the latency reported by the gauge
     */    
    public void onProbeEvent (IProbeEvent event) {
        if (event instanceof SienaActivateResponseProbeEvent) {
            Global.debug ("Got activate response ... finishing creation");
            probesDeployed ();
        }
        else if (event instanceof SienaLatencyProbeEvent) {
            SienaLatencyProbeEvent latencyEvent =
                    (SienaLatencyProbeEvent )event;
            doReporting = true;
            //Avg = alpha * curVal + (1 - alpha) * Avg
            double newLatency = new Double (latencyEvent.getProbeEvent ().getLatency ()).doubleValue ();
            synchronized (latency) {
                latency = new Double ((ALPHA * newLatency) + ((1 - ALPHA) * this.latency.doubleValue ()));
            }
  
        }
    }
    
        /** Reports a new value. This should be overridden by extenders of this gauge.
     */
    public void reportNewValue () {
        Double lLatency = null;
        synchronized (latency) {
            lLatency = latency;
        }
        gaugeBus.reportValue (getGaugeID (), "latency",
                              (String )mappings.get ("latency"),
                              lLatency.toString ());
    }
    
    
    public void processDelete (String gaugeMgrID) {
        latencyProbe.manageRemoval ();
        finalizeDeletion (gaugeMgrID);
        System.out.println("Gauge: " + getGaugeID ().gaugeName + " exiting");
        latencyProbe.shutdown ();
        try {
            Thread.sleep (2000); // some time to wait for things to calm down
        }
        catch (Exception e) {}        
        System.exit (0);
    }
    /** A command-line interface to start the gauge.
     * @param args -gt <i>gauge type</i><br>
     * -gn <i>gauge name</i><br>
     * -mt <i>model type</i><br>
     * -mn <i>model name</i><br>
     * [-setup <i>name=value</i>]<br>
     * [-mappings <i>name=property</i>]<br>
     * -creator <i>creator ID</i><br>
     * -senp <i>siena port</i><br>
     * [-debug]
     */    
    public static void main (String [] args) {

        
        String creatorID = null;
        String gaugeName = null;
        String gaugeType = null;
        String modelName = null;
        String modelType = null;
        StringPairVector mappings = new StringPairVector ();
        StringPairVector setupParams = new StringPairVector ();
        boolean gotMappings = false;
        boolean gotSetup = false;
        // Process args
        boolean ok = true;
        for (int i = 0; i < args.length && ok; i++) {
            String tag = args [i];
            if (args [i].equals ("-creator")) {
                creatorID = args [++i];
            }
            else if (args [i].equals ("-gn")) {
                gaugeName = args [++i];
            }
            else if (args [i].equals ("-gt")) {
                gaugeType = args [++i];
            }
            else if (args [i].equals ("-mn")) {
                modelName = args [++i];
            }
            else if (args [i].equals ("-mt")) {
                modelType = args [++i];
            }
            else if (args [i].equals ("-setup")) {
                gotSetup = true;
                i++;
                while (i < args.length && args [i].charAt (0) != '-') {
                    String setup = args [i];
                    String name = setup.substring (0, setup.indexOf ('='));
                    String value = setup.substring (setup.indexOf ('=') + 1);
                    setupParams.addElement (name, value);
                    i++;
                } 
                i--;
            }
            else if (args [i].equals ("-mappings")) {
                gotMappings = true;
                i++;
                while (i < args.length && args [i].charAt (0) != '-') {
                    String mapping = args [i];
                    String value = mapping.substring (0, mapping.indexOf ('='));
                    String property = mapping.substring (mapping.indexOf ('=') + 1);
                    mappings.addElement (value, property);
                    i++;
                }
                i--;
            }
            else if (args [i].equals ("-gb")) {
                String senp = args [++i];
                edu.cmu.cs.able.gaugeInfrastructure.Siena.Initialization.initSiena (senp);
 //               probeBus = new SienaProbeBus ("senp://snowgum.wv.cs.cmu.edu:7130");
            }
            else if (args [i].equals ("-pb")) {
                probeBus = new SienaProbeBus (args [++i]);
            }
            else if (args [i].equals ("-debug")) {
                edu.cmu.cs.able.gaugeInfrastructure.util.Global.debugFlag = true;
            }
            else {
                ok = false;
            }            
        }
        
        if (!ok) {
            printUsageMessage ();
            System.exit (0);
        }
        
        GaugeID gid = new GaugeID ();
        gid.modelName = modelName;
        gid.modelType = modelType;
        gid.gaugeName = gaugeName;
        gid.gaugeType = gaugeType;
   
       
        LatencyGauge gauge = new LatencyGauge (gid, creatorID, setupParams, mappings);
       
        
    }
    
    private static void printUsageMessage () {
        System.out.println ("Usage: LatencyGauge -creator <creatorID> -mt <modelType> -mn <modelName> -gt <gaugeType> -gn <gaugeName> -setup <name=value> -mappings <value=property> -senp <sienaport>");
        
    }
    
}
