/*
 * SienaTimeGauge.java
 *
 * Created on November 6, 2001, 2:29 PM
 */

package edu.cmu.cs.able.gauges.Siena;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.gaugeInfrastructure.*;
import java.util.*;
/** This gauge provides an example that periodically reports
 * the date and time to the gauge bus. Its type is:<br>
 * TimeGauge = [<br>
 * &nbsp;&nbsp;type = edu.cmu.cs.able.gauges.Siena.SienaTimeGauge,<br>
 * &nbsp;&nbsp;setup = []<br>
 * &nbsp;&nbsp;config = [frequency : int; report : boolean [default = false;]]<br>
 * &nbsp;&nbsp;values = [date : String]<br>
 *
 * @author Bradley Schmerl
 * @version 1.1
 */
public class SienaTimeGauge extends edu.cmu.cs.able.gaugeInfrastructure.Siena.SienaGauge {

    Date date;
    int frequency;
    
    protected String gaugeType = "edu.cmu.cs.able.gauges.Siena.SienaTimeGauge";
    
    /** Creates new SienaTimeGauge
     * @param gaugeID The ID of the new gauge.
     * @param creatorID The ID of the gauge manager that created the gauge.
     * @param setupParams The parameters that the gauge uses to set itself up. The
     * valid setup parameters for this gauge are:<br>
     * &nbsp;&nbsp;frequency : int
     * @param mappings The property that the <pre>date</pre> value is associated with.
     */
    public SienaTimeGauge(GaugeID gaugeID, String creatorID, StringPairVector setupParams, StringPairVector mappings) {
        super (gaugeID, creatorID, setupParams, mappings);
        
        for (int i = 0; i < setupParams.size (); i++) {
            if (setupParams.nameAt (i).equals ("frequency")) {
                frequency = Integer.parseInt (setupParams.valueAt (i));
                sleepTime = frequency; 
            }
            else if (setupParams.nameAt (i).equals ("report")) {
                if (setupParams.valueAt (i).equals ("true")) {
                    doReporting = true;
                }
            }
        }
        System.out.println (gaugeID.toString () + " created with");
        for (int i = 0; i < setupParams.size (); i ++) {
            System.out.println ("  setup " + setupParams.nameAt (i) + " = " + setupParams.valueAt (i));
        }
        
        for (int j = 0; j < mappings.size (); j++) {
            System.out.println ("  map " + mappings.nameAt (j) + " to " + mappings.valueAt (j));
        }
        
        finalizeCreation (creatorID, true);
    }

    /** This method is used to configure the gauge.
     * @param configParams The parameters used to configure the gauge. For the
     * SienaTimeGauge, the valid configuration parameters are:<br>
     * &nbsp;&nbsp;frequency : int (millisecond period of reporting)<br>
     * &nbsp;&nbsp;report : true (whether the gauge should report a value)
     * @return Whether the gauge was successfully configured.
      */    
    public boolean configure(StringPairVector configParams) {
        for (int i = 0; i < configParams.size (); i++) {
            if (configParams.nameAt (i).equals ("frequency")) {
                frequency = Integer.parseInt (configParams.valueAt (i));
                sleepTime = frequency;
            }
            else if (configParams.nameAt (i).equals ("report")) {
                if (configParams.valueAt (i).equals ("true"))
                    doReporting = true;
                else doReporting = false;
            }
        }
        return true;
    }
    
    /** Returns the current date.
     * @param values The vector into which to place the date.
     * @return Whether the gauge could report it's current value.
     */    
    public boolean queryAllValues(GaugeValueVector values) {
        values.addElement ("date", (String )mappings.get ("date"), date.toString ());
        return true;
    }
    
    /** Returns the state of the gauge.
     * @param setupParams The vector into which the setup parameters are copied. Will be
     * empty for this gauge.
     * @param configParams The configuration parameters for this gauge. Will have
     * an element for <pre>frequency</pre> and <pre>report</pre>.
     * @param mappings The property that each gauge value is associated with.
     * Will have an element for <pre>date</pre>.
     * @return Whether the state of the gauge could be queried.
     */    
    public boolean queryState(StringPairVector setupParams, StringPairVector configParams, StringPairVector mappings) {
        this.setupParams.copyInto (setupParams);
        configParams.addElement ("frequency", new Integer (frequency).toString ());
        configParams.addElement ("report", new Boolean (doReporting).toString ());
        
        Enumeration enum = this.mappings.keys ();
        while (enum.hasMoreElements ()) {
            String key = (String )enum.nextElement ();
            mappings.addElement (key, (String )this.mappings.get (key));
        }
        
        return true;
    }
    
    /** The method called to query the value of the gauge. This
     * provides a direct query for the gauge, as opposed to the
     * gauge reporting to value on the bus.
     * @param valueName The name of the value to query.
     * @return The corresponding value of the gauge. (Note: <I>it is assumed
     * that the querier know the property associated with this
     * value</I>).
     */    
    public String queryValue(String valueName) {
        if (valueName.equals ("date")) {
            return date.toString ();
        }
        else {
            return "";
        }
    }
    
    /** This method is called periodically by the reporting thread
     * to generate a new value and report it. It is inherited from
     * Siena gauge.
     */    
    public void reportNewValue() {
        date = new Date ();
        gaugeBus.reportValue (getGaugeID (), "date", (String )mappings.get ("date"), date.toString ());
    }
    
    /** A command-line interface to start the gauge.
     * @param args -gt <i>gauge type</i><br>
     * -gn <i>gauge name</i><br>
     * -mt <i>model type</i><br>
     * -mn <i>model name</i><br>
     * [-setup <i>name=value</i>]<br>
     * [-mappings <i>name=property</i>]<br>
     * -creator <i>creator ID</i><br>
     * -senp <i>siena port</i><br>
     * [-debug]
     */    
    public static void main (String [] args) {

        
        String creatorID = null;
        String gaugeName = null;
        String gaugeType = null;
        String modelName = null;
        String modelType = null;
        StringPairVector mappings = new StringPairVector ();
        StringPairVector setupParams = new StringPairVector ();
        boolean gotMappings = false;
        boolean gotSetup = false;
        // Process args
        boolean ok = true;
        for (int i = 0; i < args.length && ok; i++) {
            String tag = args [i];
            if (args [i].equals ("-creator")) {
                creatorID = args [++i];
            }
            else if (args [i].equals ("-gn")) {
                gaugeName = args [++i];
            }
            else if (args [i].equals ("-gt")) {
                gaugeType = args [++i];
            }
            else if (args [i].equals ("-mn")) {
                modelName = args [++i];
            }
            else if (args [i].equals ("-mt")) {
                modelType = args [++i];
            }
            else if (args [i].equals ("-setup")) {
                gotSetup = true;
                i++;
                while (i < args.length && args [i].charAt (0) != '-') {
                    String setup = args [i];
                    String name = setup.substring (0, setup.indexOf ('='));
                    String value = setup.substring (setup.indexOf ('=') + 1);
                    setupParams.addElement (name, value);
                    i++;
                } 
                i--;
            }
            else if (args [i].equals ("-mappings")) {
                gotMappings = true;
                i++;
                while (i < args.length && args [i].charAt (0) != '-') {
                    String mapping = args [i];
                    String value = mapping.substring (0, mapping.indexOf ('='));
                    String property = mapping.substring (mapping.indexOf ('=') + 1);
                    mappings.addElement (value, property);
                    i++;
                }
                i--;
            }
            else if (args [i].equals ("-senp")) {
                edu.cmu.cs.able.gaugeInfrastructure.Siena.Initialization.initSiena (args [++i]);
            }
            else if (args [i].equals ("-debug")) {
                edu.cmu.cs.able.gaugeInfrastructure.util.Global.debugFlag = true;
            }
            else {
                ok = false;
            }            
        }
        
        if (!ok) {
            printUsageMessage ();
            System.exit (0);
        }
        
        GaugeID gid = new GaugeID ();
        gid.modelName = modelName;
        gid.modelType = modelType;
        gid.gaugeName = gaugeName;
        gid.gaugeType = gaugeType;
   
       
        SienaTimeGauge gauge = new SienaTimeGauge (gid, creatorID, setupParams, mappings);
       
        
    }
    
    private static void printUsageMessage () {
        System.out.println ("Usage: SienaTimeGauge -creator <creatorID> -mt <modelType> -mn <modelName> -gt <gaugeType> -gn <gaugeName> -setup <name=value> -mappings <value=property> -senp <sienaport>");
        
    }
    
}
